-- some interesting-looking spiral examples, based on
-- FF effects by Greg Schorno
module Spirals where

import Pan

spiral1 :: IO()
spiral1 = spiralUI "Spirals #1" spirally

spiral2 :: IO()
spiral2 = spiralUI "Spirals #2" spiralFun2

spiralUI :: String
         -> (FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> Image ColorE)
         -> IO ()
spiralUI lab f = compileImage lab $ do
   ctl0 <- slider "Spokes 1" (1,24) 1
   ctl1 <- slider "Spokes 2" (1,24) 1
   ctl2 <- slider "Operation" (0,255) 0
   ctl3 <- slider "Twist1" (0,255) 9 -- -8,8) 0
   ctl4 <- slider "Twist2" (0,255) 9 -- -8,8) 0
   ctl5 <- slider "Red" (0,255) 0
   ctl6 <- slider "Green" (0,255) 0
   ctl7 <- slider "Blue" (0,255) 0
   return (f ctl0 ctl1 ctl2 ctl3 ctl4 ctl5 ctl6 ctl7)

spirally :: FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> Image ColorE
spirally ctl0 ctl1 ctl2 ctl3 ctl4 ctl5 ctl6 ctl7 (x,y) = (f2i b, f2i g, f2i r, 255)
 where
  (m,d) = toPolarFF (x,y)

  v0 = (ff_sin ((d + (m * val ctl3 (-8) 8)) * i2f (f2i ctl0 `mod` 1024)) + 512) / 4
  v1 = (ff_sin ((d + (m * val ctl4 (-8) 8)) * i2f (f2i ctl1 `mod` 1024)) + 512) / 4
  
  r = scl ( ctl5
  	  , 0      -- orig: 64
	  , 255    -- orig: 192
	  , clampBF $
  	    ifE (f2i ctl2 .&. 128 /=* 0)
  		(min v0 v1)
		(v0 - v1)
	  , clampBF $
	    ifE (f2i ctl2 .&. 16 /=* 0)
		(min 255 (max v0 v1))
		(max 0 (v1 - v0))
	  )

  g = scl ( ctl6
          , 0     -- orig: 64
	  , 255   -- orig: 192
	  , clampBF $
	    ifE (f2i ctl2 .&. 64 /=* 0)
		(min v0 v1)
		(v0 - v1)
	  , clampBF $
	    ifE (f2i ctl2 .&. 8 /=* 0)
		(max v0 v1)
		(v1 - v0)
	  )

  b = scl ( ctl7
          , 0      -- orig: 64
	  , 255    -- orig: 192
  	  , clampBF $
	    ifE (f2i ctl2 .&. 32 /=* 0)
	    	(min v0 v1)
		(v0 - v1)
	  , clampBF $
	    ifE (f2i ctl2 .&. 4 /=* 0)
	        (max v0 v1)
		(v1-v0)
	  )

spiralFun2 :: FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> FloatE -> Image ColorE
spiralFun2 ctl0 ctl1 ctl2 ctl3 ctl4 ctl5 ctl6 ctl7 (x,y) = (f2i b, f2i g, f2i r, 255)
 where
  (m,d) = toPolarFF (x,y)
  
  bigM = sqrt (i2f widthV * i2f widthV + i2f heightV * i2f heightV) / 2
  
  v8 :: FloatE
  v8 = m*m/(bigM*bigM/256)

  v0 = ff_sin ((d + (val ctl3 (-v8) v8))*i2f (f2i ctl0 `mod` 1024)) / 4
  v1 = ff_sin ((d + (val ctl4 (-v8) v8))*i2f (f2i ctl1 `mod` 1024)) / 4
  
  r  = scl ( ctl5
           , 64
	   , 192
	   , clampBF $
	     ifE (f2i ctl2 .&. 128 /=* 0)
	         (min v0 v1)
		 (v0-v1)
	   , clampBF $
	     ifE (f2i ctl2 .&. 16 /=* 0)
	         (max v0 v1)
		 (val ctl5 v0 (abs (v1-v0)))
           )

  g = clampBF $
      ifE (f2i ctl2 .&. 64 /=* 0)
   	   (scl ( ctl6
	        , 64
		, 192
		, min v0 v1
		, max v0 v1
		))
	   (ctl6 * abs (v0-v1)/ 128)

  b = clampBF $
      ifE (f2i ctl2 .&. 32 /=* 0)
   	   (scl ( ctl7
	        , 64
		, 192
		, min v0 v1
		, max v0 v1
		))
	   (ctl7 * abs (v0-v1)/ 128)
	   
{-
 put(m*m/(M*M/256),8)
,put((d+(val(3,-get(8),get(8))))*val(0,1,24)%D,0)
,put((sin(get(0)))/4,0)
,put((d+(val(4,-get(8),get(8))))*val(1,1,24)%D,1)
,put((sin(get(1)))/4,1)
,scl(ctl(5),64,192,ctl(2)&128?min(get(0),get(1)):get(0)-get(1),ctl(2)&16?max(get(0),get(1)):val(5,get(0),abs(get(1)-get(0))))
-}

scl :: (FloatE, FloatE, FloatE, FloatE, FloatE) -> FloatE
scl (v, il,ih, ol,oh) =
--  i2f $ 
  ifE (v >=* il &&* v <=* ih)
      (ol + (oh-ol)*((v-il) / (ih-il)))
      v

val :: FloatE -> FloatE -> FloatE -> FloatE
val v lo hi = lo + v * ((hi-lo) / 256.0)

toPolarFF :: PointE -> (FloatE, FloatE)
toPolarFF (x,y) = (m, d)
  where
   (m, d') = toPolar (x,y)
   d = d' * 512

ff_sin :: FloatE -> FloatE
ff_sin  x = 512 * (sin (x/512))

clampBF :: FloatE -> FloatE
clampBF x = min 255 (max 0 x)

widthV :: IntE
widthV = var "width"

heightV :: IntE
heightV = var "height"

