// ViewWindow.h: interface for the CViewWindow class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VIEWWINDOW_H__75FDE9DD_C83D_4746_8E0D_D95200DC22B3__INCLUDED_)
#define AFX_VIEWWINDOW_H__75FDE9DD_C83D_4746_8E0D_D95200DC22B3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MESSAGE_HANDLER_DECL(x)  LRESULT x(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)

#include "stdafx.h"
#include "WindowVirt.h"
#include "MouseUpdater.h"
#include "ImageView.h"

enum InputMode { NoInput, Panning, Zooming };

const int displayTimer = 1;

// For stochastic, accumulation-buffered anti-aliasing.  Should really be
// dynamically set by the user.
// const maxDisplayPasses = 20;

//
// CViewWindow is an input and an output surface.
//
class CViewWindow : public CWindowImpl<CViewWindow,CWindowVirt>
{
public:
    CViewWindow(int *pDisplayPasses, double *pMaxJitter,
                CMouseUpdater* pMouseUpdater, bool timeVarying);
    // CViewWindow(int *pDisplayPasses, double *pMaxJitter, CImageView* pImage, CMouseUpdater* pMouseUpdater);
    virtual ~CViewWindow();

    void SetImageView(CImageView* pImage);
    void CopyToClipboardResizing(int copyWidth, int copyPasses);

    BEGIN_MSG_MAP(CViewWindow)
        MESSAGE_HANDLER(WM_CLOSE, OnClose)
        MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
        MESSAGE_HANDLER(WM_CREATE, OnCreate)
        MESSAGE_HANDLER(WM_COMMAND, OnCommand)
        MESSAGE_HANDLER(WM_SIZE, OnSize)
        MESSAGE_HANDLER(WM_CHAR, OnChar)
        MESSAGE_HANDLER(WM_KEYDOWN, OnKeyDown)
        MESSAGE_HANDLER(WM_PAINT, OnPaint)
        MESSAGE_HANDLER(WM_MOUSEMOVE, OnMouseMove)
        MESSAGE_HANDLER(WM_LBUTTONDOWN, OnButton)
        MESSAGE_HANDLER(WM_LBUTTONUP, OnButton)
        MESSAGE_HANDLER(WM_RBUTTONDOWN, OnButton)
        MESSAGE_HANDLER(WM_RBUTTONUP, OnButton)
        MESSAGE_HANDLER(WM_MOUSEWHEEL, OnWheel)
        MESSAGE_HANDLER(WM_TIMER, OnTimer)
    END_MSG_MAP()

    MESSAGE_HANDLER_DECL(OnClose);
    MESSAGE_HANDLER_DECL(OnDestroy);
    MESSAGE_HANDLER_DECL(OnCreate);
    MESSAGE_HANDLER_DECL(OnCommand);
    MESSAGE_HANDLER_DECL(OnSize);
    MESSAGE_HANDLER_DECL(OnChar);
    MESSAGE_HANDLER_DECL(OnKeyDown);
    MESSAGE_HANDLER_DECL(OnPaint);
    MESSAGE_HANDLER_DECL(OnMouseMove);
    MESSAGE_HANDLER_DECL(OnButton);
    MESSAGE_HANDLER_DECL(OnWheel);
    MESSAGE_HANDLER_DECL(OnTimer);

    // Call Changed() whenever the view contents change.  Will set off a
    // new multi-pass rendering.  ToDo: consider merging Changed and
    // StartRendering.
    void Changed(bool firstRender=false) { StartRendering(firstRender); }

    void GetMouseInput(float &x, float &y) {
        x = m_mouseInputX;
        y = m_mouseInputY;
    }


protected:
    // After a change, start a new sequence render passes.  Maybe do an
    // initial rendering, as in a resize event.
    void StartRendering(bool firstRender) {
        m_nextPass = 1;  // Or 2 for motion blur.
        if (!m_timeVarying && firstRender && m_image)
            DoRenderPass();
        if (!m_timerRunning) {
            ::SetTimer(m_hWnd,displayTimer,30,NULL);
            m_timerRunning = true;
        }
    }
    bool NeedsRenderPass() {
        return m_nextPass <= *m_pRenderPasses;
    }
    // For non-time-varying images, render the entire window even though
    // some may be occluded.  Later, when we merely blit or perform
    // later render passes, the image will be ready.
    void DoRenderPass() {
        RECT rc;
        rc.left   = 0;
        rc.right  = m_width;
        rc.top    = 0;
        rc.bottom = m_height;
        m_image->Render(rc, m_width, m_height, m_nextPass, *m_pMaxJitter);
        m_nextPass++;
        InvalidateRect(NULL,FALSE); // trigger repaint if visible
    }
    // Done in response to diplay timer for non-time-varying images
    void TriggerRenderPass() {
        if (NeedsRenderPass()) {
            DoRenderPass();
        }
        else {
            ::KillTimer(m_hWnd,displayTimer);
            m_timerRunning = false;
        }
    }

    int m_width, m_height;
    int m_grabWindowX, m_grabWindowY;
    float m_mouseInputX, m_mouseInputY;

    double m_grabImageX, m_grabImageY;
    double m_grabScale;

    // For multi-pass rendering.
    int m_nextPass;
    int *m_pRenderPasses; double *m_pMaxJitter;

    InputMode m_uiMode;

    CImageView*     m_image;
    CMouseUpdater*  m_mouseUpdater;

    bool m_timeVarying, m_timerRunning;
};

#endif // !defined(AFX_VIEWWINDOW_H__75FDE9DD_C83D_4746_8E0D_D95200DC22B3__INCLUDED_)
