// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H__7597B73C_7EC9_41E7_9434_A085645B6C7C__INCLUDED_)
#define AFX_MAINFRM_H__7597B73C_7EC9_41E7_9434_A085645B6C7C__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "stdafx.h"

#include "Box.h"
#include "Tick.h"
#include "Button.h"
#include "RadioBox.h"
#include "FileSelector.h"
#include "Slider.h"
#include "NewDlg.h"
#include "RemoveDlg.h"
#include "ViewSettings.h"
#include "PerfData.h"
#include "StatusBar.h"

class CMainFrame : 
        public CFrameWindowImpl<CMainFrame>,
        public CUpdateUI<CMainFrame>,
		public CMessageFilter,
        public CIdleHandler
{
public:
    DECLARE_FRAME_WND_CLASS(NULL, IDR_MAINFRAME)


    CPanViewerView m_view;
    CCommandBarCtrl m_CmdBar;


    CMainFrame() : m_bar(NULL), m_copyWidth(1000),
        m_passes(40), m_copyPasses(20), m_maxJitter(0.7),
        m_view(&m_passes,&m_maxJitter) { };

    ~CMainFrame()
    {
        delete m_perfData;
        delete m_bar;
    }


    virtual BOOL PreTranslateMessage(MSG* pMsg)
    {
        if(CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg))
            return TRUE;

        return m_view.PreTranslateMessage(pMsg);
    }

    virtual BOOL OnIdle()
    {
        UIUpdateToolBar();
        return FALSE;
    }

    BEGIN_MSG_MAP(CMainFrame)
        MESSAGE_HANDLER(WM_CREATE, OnCreate)
        MESSAGE_HANDLER(WM_SIZE, OnSize)
        COMMAND_ID_HANDLER(ID_APP_EXIT, OnFileExit)
        COMMAND_ID_HANDLER(ID_FILE_NEW, OnFileNew)
        COMMAND_ID_HANDLER(ID_FILE_REMOVE, OnFileRemove)
        COMMAND_ID_HANDLER(ID_FILE_CLOSE, OnFileClose)
        COMMAND_ID_HANDLER(ID_FILE_SAVE, OnFileSave)
        COMMAND_ID_HANDLER(ID_FILE_SAVE_AS, OnFileSaveAs)
        COMMAND_ID_HANDLER(ID_VIEW_TOOLBAR, OnViewToolBar)
        COMMAND_ID_HANDLER(ID_VIEW_STATUS_BAR, OnViewStatusBar)
        COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
        COMMAND_ID_HANDLER(ID_FILE_OPEN, OnFileOpen)
        COMMAND_ID_HANDLER(ID_EDIT_COPY, OnEditCopy)
        COMMAND_ID_HANDLER(ID_VIEW_SETTINGS, OnViewSettings)
        CHAIN_MSG_MAP(CUpdateUI<CMainFrame>)
        CHAIN_MSG_MAP(CFrameWindowImpl<CMainFrame>)
    END_MSG_MAP()

    BEGIN_UPDATE_UI_MAP(CMainFrame)
            UPDATE_ELEMENT(ID_VIEW_TOOLBAR, UPDUI_MENUPOPUP)
            UPDATE_ELEMENT(ID_VIEW_STATUS_BAR, UPDUI_MENUPOPUP)
    END_UPDATE_UI_MAP()

    LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
    {
        // create command bar window
        HWND hWndCmdBar = m_CmdBar.Create(m_hWnd, rcDefault, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE);
        // atach menu
        m_CmdBar.AttachMenu(GetMenu());
        // load command bar images
        m_CmdBar.LoadImages(IDR_MAINFRAME);
        // remove old menu
        SetMenu(NULL);

        HWND hWndToolBar = CreateSimpleToolBarCtrl(m_hWnd, IDR_MAINFRAME, FALSE, ATL_SIMPLE_TOOLBAR_PANE_STYLE);

        CreateSimpleReBar(ATL_SIMPLE_REBAR_NOBORDER_STYLE);
        AddSimpleReBarBand(hWndCmdBar);
        AddSimpleReBarBand(hWndToolBar, NULL, TRUE);
        CreateSimpleStatusBar((LPCTSTR)NULL);

        m_bar = new CStatusBar(m_hWndStatusBar,3);
        m_perfData = new CPerfData(m_bar);

        m_hWndClient = m_view.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, WS_EX_CLIENTEDGE);

        UIAddToolBar(hWndToolBar);
        UISetCheck(ID_VIEW_TOOLBAR, 1);
        UISetCheck(ID_VIEW_STATUS_BAR, 1);

        CMessageLoop* pLoop = _Module.GetMessageLoop();
        pLoop->AddMessageFilter(this);
        pLoop->AddIdleHandler(this);

        // The ViewerView needs to know the HWND so that when it comes
        // to persisting/loading a settings file, the view size can be
        // stashed away too.
        m_view.SetMainHWND(m_hWnd);

        return 0;
    }

    LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
    {
        // Intercept the resize event & pass it on to the statusbar.
        if (m_bar) {
            m_bar->ResizeStatusBar();
        }

        // pass the message down the chain of msg maps.
        bHandled = false;
        return 0;
    }

    LRESULT OnFileExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
            PostMessage(WM_CLOSE);
            return 0;
    }

    LRESULT OnFileClose(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
        m_view.CloseEffect();

        // Reset window title.
        SetTitle(NULL);

        return 0;
    }

    LRESULT OnFileNew(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
        CLSID   clsid;
        PTSTR   progID = NULL;
        bool    userSelected = false;
        LPTSTR  effectName;

        CNewDlg dlg(&clsid,&progID,&userSelected, &effectName);

        if ((dlg.DoModal()) == IDOK && userSelected) {
            SIZE sz = m_view.OpenEffect(m_hWndClient, clsid,progID, m_perfData);
            if (sz.cx != 0 && sz.cy != 0) {
                ResizeFrame(sz);
                SetTitle(effectName);
                delete [] effectName;
            }
        }
        return 0;
    }

    LRESULT OnFileRemove(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
        CRemoveDlg dlg;
        int c = dlg.DoModal();
        return 0;
    }

    LRESULT OnFileSave(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
    {
        m_view.SaveEffect(FALSE);
        return 0;
    }
    LRESULT OnFileSaveAs(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
    {
        m_view.SaveEffect(TRUE);
        return 0;
    }
    
    LRESULT OnViewSettings(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
    {
        CViewSettings dlg(&m_copyWidth,&m_passes,&m_copyPasses,&m_maxJitter);
        dlg.DoModal();
        return 0;
    }

    LRESULT OnEditCopy(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
    {
        CEffect* effect = m_view.GetEffect();
        if (!effect)
            ::MessageBox(hWndCtl,TEXT("Cannot copy: no effect loaded"),TEXT("EditCopy"),MB_OK);
        else {
            effect->GetViewWindow()->CopyToClipboardResizing(m_copyWidth,m_copyPasses);
        }
        return 0;
    }

    void OpenDoc(LPTSTR fName)
    {
        LPTSTR effectName;

        SIZE sz = m_view.OpenEffect(m_hWndClient, fName, m_perfData, effectName);
        if (sz.cx != 0 && sz.cy != 0) {
            ResizeFrame(sz);
            if (m_bar) {
                m_bar->ResizeStatusBar();
            }
            SetTitle(effectName);
            delete [] effectName;
        }
    }

    void OpenComponent(CLSID& clsid, LPTSTR& effectName, LPTSTR progID)
    {

         SIZE sz = m_view.OpenEffect(m_hWndClient, clsid,progID, m_perfData);
         if (sz.cx != 0 && sz.cy != 0) {
             ResizeFrame(sz);
             SetTitle(effectName);
             delete [] effectName;
         }
    }

    LRESULT OnFileOpen(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
	    // Time to pop up the FileOpen dialog.
        OPENFILENAME ofn;
        TCHAR pstr[MAX_PATH];
	    int i = 0;

	    
	    // This *has* to be zero-terminated before passing it in
	    pstr[0] = TEXT('\0');
	      
	    ofn.lStructSize       = sizeof(OPENFILENAME);
	    ofn.hwndOwner         = GetDesktopWindow();
	    ofn.lpstrFilter       = TEXT("Pan Files (*.pan)\0*.PAN\0All Files (*.*)\0*.*\0");
	    ofn.lpstrCustomFilter = NULL;
	    ofn.nFilterIndex      = 1;
	    ofn.lpstrFile         = (LPTSTR)pstr;
	    ofn.nMaxFile          = MAX_PATH;
	    ofn.lpstrInitialDir   = NULL;
	    ofn.lpstrTitle        = NULL;
        ofn.lpstrFileTitle    = NULL;
	    ofn.lpstrDefExt       = NULL;
	    ofn.Flags             = OFN_FILEMUSTEXIST | OFN_READONLY | OFN_PATHMUSTEXIST;

	    // We're now ready to ask for the selection:
        if ( !GetOpenFileName(&ofn) ) {
            TCHAR		sErrMsg[256];
	        long		errMsg;
	    
	        errMsg = CommDlgExtendedError();
	    
	        if (errMsg == 0) { // Did the user Cancel?
	           return 0;
	        } else {
	           _stprintf(sErrMsg,TEXT("Error %d returned from GetOpenFileName()"), errMsg);
	           MessageBox(sErrMsg, TEXT("FileOpen"), MB_OK);
	           return 0;
            } 
        } else {
            OpenDoc(pstr);
            return 0;
        }
    }



    LRESULT OnViewToolBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
        static BOOL bNew = TRUE;	// initially visible
        bNew = !bNew;
        ::SendMessage(m_hWndToolBar, RB_SHOWBAND, 1, bNew);	// toolbar is band #1
        UISetCheck(ID_VIEW_TOOLBAR, bNew);
        UpdateLayout();
        return 0;
    }

    LRESULT OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
        BOOL bNew = !::IsWindowVisible(m_hWndStatusBar);
        ::ShowWindow(m_hWndStatusBar, bNew ? SW_SHOWNOACTIVATE : SW_HIDE);
        UISetCheck(ID_VIEW_STATUS_BAR, bNew);
        UpdateLayout();
        return 0;
    }

    LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
        CAboutDlg dlg;
        dlg.DoModal();
        return 0;
    }

private:
    CStatusBar* m_bar;
    CPerfData*  m_perfData;

    void SetTitle(LPCTSTR szTitle)
    {
        LPTSTR titleString;
        TCHAR  appTitle[1024];
        int len;

        len = LoadString(NULL,IDR_MAINFRAME,appTitle, 1024);
        appTitle[len] = TEXT('\0');

        if (szTitle == NULL) {
            SetWindowText(appTitle);
        } else {
            titleString = new TCHAR[_tcslen(szTitle) + _tcslen(appTitle) + 10];
            _stprintf(titleString, TEXT("%s - %s"), szTitle, appTitle);
            SetWindowText(titleString);

            delete [] titleString;
        }
    }
    //
    // METHOD: ResizeFrame
    //
    // COMMENTS: After a new Pan image UI has been opened up, the
    //           size of the viewer needs to be adjusted to at least
    //           fit the natural sizes of this new UI.
    // 
    void ResizeFrame(SIZE sz)
    {
        // Given the size of the Pan image UI, adjust the size of the frame and
        // view area window. The view area's width is the max of the current width
        // and the nat. width of the Pan UI (ditto for height.)
        // 
        // The frame is resized accordingly, but we take into account the size needed
        // to display the command & status bars (+ window title.)

        if (sz.cx != 0 && sz.cy != 0) {
            RECT r1, r2;

            ::GetWindowRect(m_hWnd,&r1);
            ::GetClientRect(m_hWndClient,&r2);
            ::SetWindowPos(m_hWndClient,NULL, 0,0,
                           max(sz.cx, r2.right - r2.left),
                           max(r2.bottom - r2.top, sz.cy), SWP_NOMOVE);
            ::SetWindowPos(m_hWnd, NULL, 0,0,
                           max(sz.cx, r1.right - r1.left),
                           max(r1.bottom - r1.top,
                               sz.cy + 10 + ((r1.bottom - r1.top) - (r2.bottom - r2.top))), SWP_NOMOVE);
        }
    }

    int m_copyWidth;  // for scaling during "copy" command
    // For multi-pass, anti-aliased display:
    int m_passes, m_copyPasses; double m_maxJitter;
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINFRM_H__7597B73C_7EC9_41E7_9434_A085645B6C7C__INCLUDED_)
